﻿using System;
using System.Windows;
using System.Windows.Controls;
using OpcLabs.EasyOpc.UA;
using OpcLabs.EasyOpc.UA.OperationModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using OpcLabs.EasyOpc.UA.LiveMapping;
using OpcLabs.EasyOpc.UA.Navigation;

namespace OPCTest5
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public class ViewModel:INotifyPropertyChanged
    {
        readonly Uri uri = new Uri("opc.tcp://127.0.0.1:49320");

        private string _err;
        public string Errors
        {
            get => _err;
            set { _err = value; OnPropertyChanged(); }
        }

        private void ClientOnDataChangeNotification(object sender, EasyUADataChangeNotificationEventArgs eventArgs)
        {
            // We have passed in a reference to the target TextBox as the State property in EasyUAMonitoredItemArguments.
            //var textBox = (TextBox)eventArgs.Arguments.State;

            if (eventArgs.Succeeded)
            {
                var statusnode = (UAAttributeData)eventArgs.Arguments.State;
                statusnode.ServerTimestamp = eventArgs.AttributeData.ServerTimestamp;
                Errors = eventArgs.AttributeData.DisplayValue();            
            }
            else
            {
                Errors = "*** Error: " + eventArgs.Exception;
            }
        }
        public UAAttributeData test;
        public ViewModel()
        {

            //test = new UAAttributeData();
            // Define the target controls, the OPC data we want to monitor, and how fast.
            //var arguments = new[]
            {
                //new EasyUAMonitoredItemArguments(test, uri,"ns=2;s=Channel1.Device1.Tag1",100)
            //    //new EasyUAMonitoredItemArguments(test, "http://opcua.demo-this.com:51211/UA/SampleServer", "nsu=http://test.org/UA/Data/;i=10845", 1000)
            //    //new EasyUAMonitoredItemArguments(TextBox2, "http://opcua.demo-this.com:51211/UA/SampleServer", "nsu=http://test.org/UA/Data/;i=10853", 1000),
            //    //new EasyUAMonitoredItemArguments(TextBox3, "http://opcua.demo-this.com:51211/UA/SampleServer", "nsu=http://test.org/UA/Data/;i=10855", 1000)
            };

            // Hook the event handler, and subscribe to OPC data
            //_client.DataChangeNotification += ClientOnDataChangeNotification;
            //_handles = _client.SubscribeMultipleMonitoredItems(arguments);
            //var data = _client.ReadValue(uri, new UANodeDescriptor("i=2255"));
            //Errors = string.Join("\n", data);
            Map();
        }

        ~ViewModel()
        {
            // Unsubscribe from OPC data, and unhook the event handler
            _client.UnsubscribeMultipleMonitoredItems(_handles);
            _client.DataChangeNotification -= ClientOnDataChangeNotification;
            _mapper.Dispose();
            _client.Dispose();
        }

        private readonly EasyUAClient _client = new EasyUAClient();

        private UAClientMapper _mapper = new UAClientMapper();

        private int[] _handles;

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged([CallerMemberName] string property = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(property));
        }

        private UAAttributeData _status;
        public UAAttributeData Status
        {
            get => _status;
            set { _status = value; OnPropertyChanged(); }
        }

        private OPCDevice _dev;
        public OPCDevice Device
        {
            get => _dev;
            set { _dev = value; OnPropertyChanged(); }
        }

        private void Map()
        {
            UAEndpointDescriptor endpointDescriptor = uri;
            Device = new OPCDevice();
            _mapper.Map(Device, new UAMappingContext
            {
                EndpointDescriptor = endpointDescriptor,
                // The NodeDescriptor below determines where in the OPC address space we want to map our data to.
                NodeDescriptor = new UANodeDescriptor
                {
                    // '#' is a reserved character in a browse name, and must be escaped by '&' in the path below.
                    BrowsePath = UABrowsePath.Parse("[ObjectsFolder].Channel1.Device1", "KEPServerEX")
                },
                MonitoringParameters = 1000,  // requested sampling interval (for subscriptions)
            }); 
            _mapper.Subscribe(true); 
        }
    }
}

